@@ -4,12 +4,15 @@ import os.path
 
 from typing import List, Sequence, Set, Tuple, Optional, Dict
 
-from mypy.build import BuildSource, PYTHON_EXTENSIONS
+from mypy.modulefinder import BuildSource, PYTHON_EXTENSIONS
 from mypy.fscache import FileSystemCache
 from mypy.options import Options
 
+MYPY = False
+if MYPY:
+    from typing_extensions import Final
 
-PY_EXTENSIONS = tuple(PYTHON_EXTENSIONS)
+PY_EXTENSIONS = tuple(PYTHON_EXTENSIONS)  # type: Final
 
 
 class InvalidSourceList(Exception):
