@@ -20,34 +20,48 @@ from mypy.types import (
 )
 from mypy.typevars import fill_typevars
 
+MYPY = False
+if MYPY:
+    from typing_extensions import Final
+
 
 # The names of the different functions that create classes or arguments.
 attr_class_makers = {
     'attr.s',
     'attr.attrs',
     'attr.attributes',
-}
+}  # type: Final
 attr_dataclass_makers = {
     'attr.dataclass',
-}
+}  # type: Final
 attr_attrib_makers = {
     'attr.ib',
     'attr.attrib',
     'attr.attr',
-}
+}  # type: Final
+
+
+class Converter:
+    """Holds information about a `converter=` argument"""
+
+    def __init__(self,
+                 name: Optional[str] = None,
+                 is_attr_converters_optional: bool = False) -> None:
+        self.name = name
+        self.is_attr_converters_optional = is_attr_converters_optional
 
 
 class Attribute:
     """The value of an attr.ib() call."""
 
     def __init__(self, name: str, info: TypeInfo,
-                 has_default: bool, init: bool, converter_name: Optional[str],
+                 has_default: bool, init: bool, converter: Converter,
                  context: Context) -> None:
         self.name = name
         self.info = info
         self.has_default = has_default
         self.init = init
-        self.converter_name = converter_name
+        self.converter = converter
         self.context = context
 
     def argument(self, ctx: 'mypy.plugin.ClassDefContext') -> Argument:
@@ -55,13 +69,13 @@ class Attribute:
         assert self.init
         init_type = self.info[self.name].type
 
-        if self.converter_name:
+        if self.converter.name:
             # When a converter is set the init_type is overridden by the first argument
             # of the converter method.
-            converter = lookup_qualified_stnode(ctx.api.modules, self.converter_name, True)
+            converter = lookup_qualified_stnode(ctx.api.modules, self.converter.name, True)
             if not converter:
                 # The converter may be a local variable. Check there too.
-                converter = ctx.api.lookup_qualified(self.converter_name, self.info, True)
+                converter = ctx.api.lookup_qualified(self.converter.name, self.info, True)
 
             # Get the type of the converter.
             converter_type = None
@@ -90,10 +104,16 @@ class Attribute:
                 if types:
                     args = UnionType.make_simplified_union(types)
                     init_type = ctx.api.anal_type(args)
+
+            if self.converter.is_attr_converters_optional and init_type:
+                # If the converter was attr.converter.optional(type) then add None to
+                # the allowed init_type.
+                init_type = UnionType.make_simplified_union([init_type, NoneTyp()])
+
             if not init_type:
                 ctx.api.fail("Cannot determine __init__ type from converter", self.context)
                 init_type = AnyType(TypeOfAny.from_error)
-        elif self.converter_name == '':
+        elif self.converter.name == '':
             # This means we had a converter but it's not of a type we can infer.
             # Error was shown in _get_converter_name
             init_type = AnyType(TypeOfAny.from_error)
@@ -122,7 +142,8 @@ class Attribute:
             'name': self.name,
             'has_default': self.has_default,
             'init': self.init,
-            'converter_name': self.converter_name,
+            'converter_name': self.converter.name,
+            'converter_is_attr_converters_optional': self.converter.is_attr_converters_optional,
             'context_line': self.context.line,
             'context_column': self.context.column,
         }
@@ -135,7 +156,7 @@ class Attribute:
             info,
             data['has_default'],
             data['init'],
-            data['converter_name'],
+            Converter(data['converter_name'], data['converter_is_attr_converters_optional']),
             Context(line=data['context_line'], column=data['context_column'])
         )
 
@@ -318,7 +339,7 @@ def _attribute_from_auto_attrib(ctx: 'mypy.plugin.ClassDefContext',
     """Return an Attribute for a new type assignment."""
     # `x: int` (without equal sign) assigns rvalue to TempNode(AnyType())
     has_rhs = not isinstance(rvalue, TempNode)
-    return Attribute(lhs.name, ctx.cls.info, has_rhs, True, None, stmt)
+    return Attribute(lhs.name, ctx.cls.info, has_rhs, True, Converter(), stmt)
 
 
 def _attribute_from_attrib_maker(ctx: 'mypy.plugin.ClassDefContext',
@@ -373,31 +394,42 @@ def _attribute_from_attrib_maker(ctx: 'mypy.plugin.ClassDefContext',
     elif convert:
         ctx.api.fail("convert is deprecated, use converter", rvalue)
         converter = convert
-    converter_name = _get_converter_name(ctx, converter)
+    converter_info = _parse_converter(ctx, converter)
 
-    return Attribute(lhs.name, ctx.cls.info, attr_has_default, init, converter_name, stmt)
+    return Attribute(lhs.name, ctx.cls.info, attr_has_default, init, converter_info, stmt)
 
 
-def _get_converter_name(ctx: 'mypy.plugin.ClassDefContext',
-                        converter: Optional[Expression]) -> Optional[str]:
-    """Return the full name of the converter if it exists and is a simple function."""
+def _parse_converter(ctx: 'mypy.plugin.ClassDefContext',
+                     converter: Optional[Expression]) -> Converter:
+    """Return the Converter object from an Expression."""
     # TODO: Support complex converters, e.g. lambdas, calls, etc.
     if converter:
         if isinstance(converter, RefExpr) and converter.node:
             if (isinstance(converter.node, FuncBase)
                     and converter.node.type
                     and isinstance(converter.node.type, FunctionLike)):
-                return converter.node.fullname()
+                return Converter(converter.node.fullname())
             elif isinstance(converter.node, TypeInfo):
-                return converter.node.fullname()
+                return Converter(converter.node.fullname())
+
+        if (isinstance(converter, CallExpr)
+                and isinstance(converter.callee, RefExpr)
+                and converter.callee.fullname == "attr.converters.optional"
+                and converter.args
+                and converter.args[0]):
+            # Special handling for attr.converters.optional(type)
+            # We extract the type and add make the init_args Optional in Attribute.argument
+            argument = _parse_converter(ctx, converter.args[0])
+            argument.is_attr_converters_optional = True
+            return argument
 
         # Signal that we have an unsupported converter.
         ctx.api.fail(
             "Unsupported converter, only named functions and types are currently supported",
             converter
         )
-        return ''
-    return None
+        return Converter('')
+    return Converter(None)
 
 
 def _parse_assignments(
@@ -430,7 +462,7 @@ def _add_cmp(ctx: 'mypy.plugin.ClassDefContext', adder: 'MethodAdder') -> None:
     #    AT = TypeVar('AT')
     #    def __lt__(self: AT, other: AT) -> bool
     # This way comparisons with subclasses will work correctly.
-    tvd = TypeVarDef('AT', 'AT', 1, [], object_type)
+    tvd = TypeVarDef('AT', 'AT', -1, [], object_type)
     tvd_type = TypeVarType(tvd)
     args = [Argument(Var('other', tvd_type), tvd_type, None, ARG_POS)]
     for method in ['__lt__', '__le__', '__gt__', '__ge__']:
@@ -463,23 +495,6 @@ def _add_init(ctx: 'mypy.plugin.ClassDefContext', attributes: List[Attribute],
         [attribute.argument(ctx) for attribute in attributes if attribute.init],
         NoneTyp()
     )
-    for stmt in ctx.cls.defs.body:
-        # The type of classmethods will be wrong because it's based on the parent's __init__.
-        # Set it correctly.
-        if isinstance(stmt, Decorator) and stmt.func.is_class:
-            func_type = stmt.func.type
-            if isinstance(func_type, CallableType):
-                func_type.arg_types[0] = ctx.api.class_type(ctx.cls.info)
-        if isinstance(stmt, OverloadedFuncDef) and stmt.is_class:
-            func_type = stmt.type
-            if isinstance(func_type, Overloaded):
-                class_type = ctx.api.class_type(ctx.cls.info)
-                for item in func_type.items():
-                    item.arg_types[0] = class_type
-                if stmt.impl is not None:
-                    assert isinstance(stmt.impl, Decorator)
-                    if isinstance(stmt.impl.func.type, CallableType):
-                        stmt.impl.func.type.arg_types[0] = class_type
 
 
 class MethodAdder:
