@@ -44,7 +44,7 @@ from mypy.nodes import (
     Node, FuncDef, NameExpr, MemberExpr, RefExpr, MypyFile, FuncItem, ClassDef, AssignmentStmt,
     ImportFrom, Import, TypeInfo, SymbolTable, Var, CallExpr, Decorator, OverloadedFuncDef,
     SuperExpr, UNBOUND_IMPORTED, GDEF, MDEF, IndexExpr, SymbolTableNode, ImportAll, TupleExpr,
-    ListExpr, ForStmt
+    ListExpr, ForStmt, Block
 )
 from mypy.semanal_shared import create_indirect_imported_name
 from mypy.traverser import TraverserVisitor
@@ -52,7 +52,7 @@ from mypy.types import CallableType
 from mypy.typestate import TypeState
 
 
-def strip_target(node: Union[MypyFile, FuncItem, OverloadedFuncDef]) -> None:
+def strip_target(node: Union[MypyFile, FuncDef, OverloadedFuncDef]) -> None:
     """Reset a fine-grained incremental target to state after semantic analysis pass 1.
 
     NOTE: Currently we opportunistically only reset changes that are known to otherwise
@@ -83,6 +83,11 @@ class NodeStripVisitor(TraverserVisitor):
         self.recurse_into_functions = False
         file_node.accept(self)
 
+    def visit_block(self, b: Block) -> None:
+        if b.is_unreachable:
+            return
+        super().visit_block(b)
+
     def visit_class_def(self, node: ClassDef) -> None:
         """Strip class body and type info, but don't strip methods."""
         self.strip_type_info(node.info)
@@ -111,6 +116,8 @@ class NodeStripVisitor(TraverserVisitor):
             return
         node.expanded = []
         node.type = node.unanalyzed_type
+        # All nodes are non-final after the first pass.
+        node.is_final = False
         # Type variable binder binds tvars before the type is analyzed.
         # It should be refactored, before that we just undo this change here.
         # TODO: this will be not necessary when #4814 is fixed.
@@ -125,15 +132,17 @@ class NodeStripVisitor(TraverserVisitor):
         for expr in node.decorators:
             expr.accept(self)
         if self.recurse_into_functions:
+            # Only touch the final status if we re-process
+            # a method target
+            node.var.is_final = False
             node.func.accept(self)
 
     def visit_overloaded_func_def(self, node: OverloadedFuncDef) -> None:
         if not self.recurse_into_functions:
             return
-        if node.impl:
-            # Revert change made during semantic analysis pass 2.
-            assert node.items[-1] is not node.impl
-            node.items.append(node.impl)
+        # Revert change made during semantic analysis pass 2.
+        node.items = node.unanalyzed_items.copy()
+        node.is_final = False
         super().visit_overloaded_func_def(node)
 
     @contextlib.contextmanager
@@ -162,6 +171,7 @@ class NodeStripVisitor(TraverserVisitor):
 
     def visit_assignment_stmt(self, node: AssignmentStmt) -> None:
         node.type = node.unanalyzed_type
+        node.is_final_def = False
         if self.type and not self.is_class_body:
             for lvalue in node.lvalues:
                 self.process_lvalue_in_method(lvalue)
@@ -249,6 +259,7 @@ class NodeStripVisitor(TraverserVisitor):
         # [*] although we always strip type, thus returning the Var to the state after pass 1.
         if isinstance(node.node, Var):
             node.node.type = None
+            self._reset_var_final_flags(node.node)
 
     def visit_member_expr(self, node: MemberExpr) -> None:
         self.strip_ref_expr(node)
@@ -263,8 +274,16 @@ class NodeStripVisitor(TraverserVisitor):
             # definition.
             self.strip_class_attr(node.name)
             node.def_var = None
+        if isinstance(node.node, Var):
+            self._reset_var_final_flags(node.node)
         super().visit_member_expr(node)
 
+    def _reset_var_final_flags(self, v: Var) -> None:
+        v.is_final = False
+        v.final_unset_in_class = False
+        v.final_set_in_init = False
+        v.final_value = None
+
     def visit_index_expr(self, node: IndexExpr) -> None:
         node.analyzed = None  # was a type alias
         super().visit_index_expr(node)
