@@ -8,6 +8,7 @@ from typing import Any, Dict, Set, Tuple, Optional
 MYPY = False
 if MYPY:
     from typing import ClassVar
+    from typing_extensions import Final
 from mypy.nodes import TypeInfo
 from mypy.types import Instance
 from mypy.server.trigger import make_trigger
@@ -38,7 +39,7 @@ class TypeState:
     # instances of the given TypeInfo. The cache also keeps track of the specific
     # *kind* of subtyping relationship, which we represent as an arbitrary hashable tuple.
     # We need the caches, since subtype checks for structural types are very slow.
-    _subtype_caches = {}  # type: ClassVar[SubtypeCache]
+    _subtype_caches = {}  # type: Final[SubtypeCache]
 
     # This contains protocol dependencies generated after running a full build,
     # or after an update. These dependencies are special because:
@@ -59,13 +60,13 @@ class TypeState:
     # of type a.A to a function expecting something compatible with protocol p.P,
     # we'd have 'a.A' -> {'p.P', ...} in the map. This map is flushed after every incremental
     # update.
-    _attempted_protocols = {}  # type: ClassVar[Dict[str, Set[str]]]
+    _attempted_protocols = {}  # type: Final[Dict[str, Set[str]]]
     # We also snapshot protocol members of the above protocols. For example, if we pass
     # a value of type a.A to a function expecting something compatible with Iterable, we'd have
     # 'a.A' -> {'__iter__', ...} in the map. This map is also flushed after every incremental
     # update. This map is needed to only generate dependencies like <a.A.__iter__> -> <a.A>
     # instead of a wildcard to avoid unnecessarily invalidating classes.
-    _checked_against_members = {}  # type: ClassVar[Dict[str, Set[str]]]
+    _checked_against_members = {}  # type: Final[Dict[str, Set[str]]]
     # TypeInfos that appeared as a left type (subtype) in a subtype check since latest
     # dependency snapshot update. This is an optimisation for fine grained mode; during a full
     # run we only take a dependency snapshot at the very end, so this set will contain all
@@ -73,59 +74,66 @@ class TypeState:
     # dependencies generated from (typically) few TypeInfos that were subtype-checked
     # (i.e. appeared as r.h.s. in an assignment or an argument in a function call in
     # a re-checked target) during the update.
-    _rechecked_types = set()  # type: ClassVar[Set[TypeInfo]]
+    _rechecked_types = set()  # type: Final[Set[TypeInfo]]
 
-    @classmethod
-    def reset_all_subtype_caches(cls) -> None:
+    # N.B: We do all of the accesses to these properties through
+    # TypeState, instead of making these classmethods and accessing
+    # via the cls parameter, since mypyc can optimize accesses to
+    # Final attributes of a directly referenced type.
+
+    @staticmethod
+    def reset_all_subtype_caches() -> None:
         """Completely reset all known subtype caches."""
-        cls._subtype_caches = {}
+        TypeState._subtype_caches.clear()
 
-    @classmethod
-    def reset_subtype_caches_for(cls, info: TypeInfo) -> None:
+    @staticmethod
+    def reset_subtype_caches_for(info: TypeInfo) -> None:
         """Reset subtype caches (if any) for a given supertype TypeInfo."""
-        if info in cls._subtype_caches:
-            cls._subtype_caches[info].clear()
+        if info in TypeState._subtype_caches:
+            TypeState._subtype_caches[info].clear()
 
-    @classmethod
-    def reset_all_subtype_caches_for(cls, info: TypeInfo) -> None:
+    @staticmethod
+    def reset_all_subtype_caches_for(info: TypeInfo) -> None:
         """Reset subtype caches (if any) for a given supertype TypeInfo and its MRO."""
         for item in info.mro:
-            cls.reset_subtype_caches_for(item)
+            TypeState.reset_subtype_caches_for(item)
 
-    @classmethod
-    def is_cached_subtype_check(cls, kind: SubtypeKind, left: Instance, right: Instance) -> bool:
+    @staticmethod
+    def is_cached_subtype_check(kind: SubtypeKind, left: Instance, right: Instance) -> bool:
         info = right.type
-        if info not in cls._subtype_caches:
+        if info not in TypeState._subtype_caches:
             return False
-        cache = cls._subtype_caches[info]
+        cache = TypeState._subtype_caches[info]
         if kind not in cache:
             return False
         return (left, right) in cache[kind]
 
-    @classmethod
-    def record_subtype_cache_entry(cls, kind: SubtypeKind,
+    @staticmethod
+    def record_subtype_cache_entry(kind: SubtypeKind,
                                    left: Instance, right: Instance) -> None:
-        cache = cls._subtype_caches.setdefault(right.type, dict())
+        cache = TypeState._subtype_caches.setdefault(right.type, dict())
         cache.setdefault(kind, set()).add((left, right))
 
-    @classmethod
-    def reset_protocol_deps(cls) -> None:
+    @staticmethod
+    def reset_protocol_deps() -> None:
         """Reset dependencies after a full run or before a daemon shutdown."""
-        cls.proto_deps = {}
-        cls._attempted_protocols = {}
-        cls._checked_against_members = {}
-        cls._rechecked_types = set()
+        TypeState.proto_deps = {}
+        TypeState._attempted_protocols.clear()
+        TypeState._checked_against_members.clear()
+        TypeState._rechecked_types.clear()
 
-    @classmethod
-    def record_protocol_subtype_check(cls, left_type: TypeInfo, right_type: TypeInfo) -> None:
+    @staticmethod
+    def record_protocol_subtype_check(left_type: TypeInfo, right_type: TypeInfo) -> None:
         assert right_type.is_protocol
-        cls._rechecked_types.add(left_type)
-        cls._attempted_protocols.setdefault(left_type.fullname(), set()).add(right_type.fullname())
-        cls._checked_against_members.setdefault(left_type.fullname(),
-                                                set()).update(right_type.protocol_members)
+        TypeState._rechecked_types.add(left_type)
+        TypeState._attempted_protocols.setdefault(
+            left_type.fullname(), set()).add(right_type.fullname())
+        TypeState._checked_against_members.setdefault(
+            left_type.fullname(),
+            set()).update(right_type.protocol_members)
 
-    @classmethod
-    def _snapshot_protocol_deps(cls) -> Dict[str, Set[str]]:
+    @staticmethod
+    def _snapshot_protocol_deps() -> Dict[str, Set[str]]:
         """Collect protocol attribute dependencies found so far from registered subtype checks.
 
         There are three kinds of protocol dependencies. For example, after a subtype check:
@@ -155,8 +163,8 @@ class TypeState:
         'subtypes.is_protocol_implementation').
         """
         deps = {}  # type: Dict[str, Set[str]]
-        for info in cls._rechecked_types:
-            for attr in cls._checked_against_members[info.fullname()]:
+        for info in TypeState._rechecked_types:
+            for attr in TypeState._checked_against_members[info.fullname()]:
                 # The need for full MRO here is subtle, during an update, base classes of
                 # a concrete class may not be reprocessed, so not all <B.x> -> <C.x> deps
                 # are added.
@@ -166,7 +174,7 @@ class TypeState:
                         # TODO: avoid everything from typeshed
                         continue
                     deps.setdefault(trigger, set()).add(make_trigger(info.fullname()))
-            for proto in cls._attempted_protocols[info.fullname()]:
+            for proto in TypeState._attempted_protocols[info.fullname()]:
                 trigger = make_trigger(info.fullname())
                 if 'typing' in trigger or 'builtins' in trigger:
                     continue
@@ -179,36 +187,38 @@ class TypeState:
                 deps.setdefault(trigger, set()).add(proto)
         return deps
 
-    @classmethod
-    def update_protocol_deps(cls, second_map: Optional[Dict[str, Set[str]]] = None) -> None:
+    @staticmethod
+    def update_protocol_deps(second_map: Optional[Dict[str, Set[str]]] = None) -> None:
         """Update global protocol dependency map.
 
         We update the global map incrementally, using a snapshot only from recently
         type checked types. If second_map is given, update it as well. This is currently used
         by FineGrainedBuildManager that maintains normal (non-protocol) dependencies.
         """
-        assert cls.proto_deps is not None, "This should not be called after failed cache load"
-        new_deps = cls._snapshot_protocol_deps()
+        assert TypeState.proto_deps is not None, (
+            "This should not be called after failed cache load")
+        new_deps = TypeState._snapshot_protocol_deps()
         for trigger, targets in new_deps.items():
-            cls.proto_deps.setdefault(trigger, set()).update(targets)
+            TypeState.proto_deps.setdefault(trigger, set()).update(targets)
         if second_map is not None:
             for trigger, targets in new_deps.items():
                 second_map.setdefault(trigger, set()).update(targets)
-        cls._rechecked_types.clear()
-        cls._attempted_protocols.clear()
-        cls._checked_against_members.clear()
+        TypeState._rechecked_types.clear()
+        TypeState._attempted_protocols.clear()
+        TypeState._checked_against_members.clear()
 
-    @classmethod
-    def add_all_protocol_deps(cls, deps: Dict[str, Set[str]]) -> None:
+    @staticmethod
+    def add_all_protocol_deps(deps: Dict[str, Set[str]]) -> None:
         """Add all known protocol dependencies to deps.
 
         This is used by tests and debug output, and also when passing
         all collected or loaded dependencies on to FineGrainedBuildManager
         in its __init__.
         """
-        cls.update_protocol_deps()  # just in case
-        assert cls.proto_deps is not None, "This should not be called after failed cache load"
-        for trigger, targets in cls.proto_deps.items():
+        TypeState.update_protocol_deps()  # just in case
+        assert TypeState.proto_deps is not None, (
+            "This should not be called after failed cache load")
+        for trigger, targets in TypeState.proto_deps.items():
             deps.setdefault(trigger, set()).update(targets)
 
 
